#
# aplicação que mostra propriedades de estimadores da média de uma distribuição
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(ggplot2)

server <- function(input, output) {

  # Desvio padrão da distribuição normal   
  st_dev <- reactive ({
      input$st_dev
  })

  # Média da distribuição normal   
  mu <- reactive ({
      input$mean
  })

  # número de amostras
  namostras <- reactive ({
      input$namostras
  })
  
  # Tamanho de cada amostra
  sample_size <- reactive ({
      input$sample_size
  })
  
  # número de observações
  n_obs <- reactive ({
      namostras() * sample_size()
  })
  
  # limite inferior do eixo x para plotar o histograma 
  min_xaxis <- reactive ({
      mu() - 4 * st_dev()
  })

  # limite superior do eixo x para plotar o histograma 
  max_xaxis <- reactive ({
      mu() + 4 * st_dev()
  })
  
  # plota a média acumulada
  output$cum_mean <- renderPlot({
      x <- rnorm(n_obs(), mu(), st_dev())
      y <- c(rep(0, n_obs()))
      # y vai acumulando a média à medida que o número de observações aumenta
      for (i in 1:n_obs()) {
          y[i] <- mean(x[1:i])
      }
      g <- ggplot(data.frame(x = 1:n_obs(), y = y), aes(x = x, y = y))
      g <- g + geom_hline(yintercept = 0) + geom_line(size = 2) + geom_hline(yintercept = mu(), col="red") + geom_line(size = 2)
      g <- g + labs(x = "Número de observações", y = "Média acumulada")
      g
      })

  # plota o primeiro elemento da amostra acumulado
  output$cum_first_element <- renderPlot({
      # y vai acumulando o primeiro elemento da amostra à medida que o número de observações aumenta
      # é equivalente a extrair uma amostra de tamanho n_obs
      y <- rnorm(n_obs(), mu(), st_dev())
      g <- ggplot(data.frame(x = 1:n_obs(), y = y), aes(x = x, y = y))
      g <- g + geom_hline(yintercept = 0) + geom_line(size = 2) + geom_hline(yintercept = mu(), col="red") + geom_line(size = 2)
      g <- g + labs(x = "Número de observações", y = "Primeiro elemento da amostra acumulado")
      g
  })

  #plota o histograma da média amostral
  output$mean_hist <- renderPlot({
      y <- c(rep(0, namostras()))
      for (i in 1:namostras()) {
          x <- rnorm(sample_size(), mu(), st_dev())
          y[i] <- mean(x)
      }
      hist(y, xlim=c(min_xaxis(), max_xaxis()), main = '', xlab = "média amostral", ylab = "Frequência")
      abline(v=c(mu(), mean(y)), col=c("red", "blue"), lw = 3)
  })
  
  #plota o histograma do primeiro elemento da amostra
  output$sample_first_element_hist  <- renderPlot({
      y <- c(rep(0, n_obs()))
      y <- rnorm(namostras(), mu(), st_dev())
      hist(y, xlim=c(min_xaxis(), max_xaxis()), main = '', xlab = "primeiro elemento da amostra", ylab = "Frequência")
      abline(v=c(mu(), mean(y)), col=c("red", "blue"), lw = 3)
  })
  
  }

ui <- fluidPage(
  
  titlePanel(expression("Propriedades de estimadores da média de uma distribuição normal")),
  
  sidebarLayout(
    sidebarPanel(
    sliderInput("namostras", 
                label = expression("Número de amostras"), 
                min = 10, max = 10000, step = 1,
                value = 500
    ),
    sliderInput("sample_size", 
                label = "Tamanho da amostra", 
                min = 1, max = 100, step = 1,
                value = 4
    ),
    sliderInput("mean", 
                label = expression("Média da distribuicao normal"), 
                min = 0, max = 40, step = 1,
                value = 20
    ),
    sliderInput("st_dev", 
                label = expression("Desvio padrão da distribuição normal"), 
                min = 1, max = 20, step = 1,
                value = 4
    ),
    submitButton("Submeter")
    ),
    
    mainPanel(
        fluidRow(
            column(4, plotOutput("cum_mean")),
            column(4, plotOutput("cum_first_element"))
        ),
        fluidRow(
            column(4, plotOutput("mean_hist")),
            column(4, plotOutput("sample_first_element_hist"))
        )
    )
  )
)


shinyApp(ui = ui, server = server)