#
# aplicação que mostra propriedades de estimadores da variância de uma distribuição
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(ggplot2)

server <- function(input, output) {

  # Desvio padrão da distribuição normal   
  st_dev <- reactive ({
      input$st_dev
  })
  
  # Média da distribuição normal   
  mu <- reactive ({
      input$mean
  })

  # número de amostras
  n_amostras <- reactive ({
      input$n
  })
  
  # Tamanho de cada amostra
  sample_size <- reactive ({
      input$sample_size
  })
  
  # número de observações
  n_obs <- reactive ({
      n_amostras() * sample_size()
  })
  
  # limite superior do eixo x para plotar o histograma 
  max <- reactive ({
      st_dev()^2  + 4 * st_dev()^2 
  })
  
  # plota a variância não tendenciosa acumulada
  output$cum_variance_unbiased  <- renderPlot({
      x <- rnorm(n_obs(), mu(), st_dev())
      y <- c(rep(0, n_obs()-1))
      # y vai acumulando a variância à medida que o número de observações aumenta
      for (i in 2:n_obs()) {
          y[i-1] <- var(x[1:i])
      }
      g <- ggplot(data.frame(x = 1:(n_obs()-1), y = y), aes(x = x, y = y))
      g <- g + geom_hline(yintercept = 0) + geom_line(size = 2) + geom_hline(yintercept = st_dev()^2, col="red") + geom_line(size = 2)
      g <- g + labs(x = "Número de observações", y = "Estimador não tendencioso da variância acumulado")
      g
      })

  # plota a variância tendenciosa acumulada
  output$cum_variance_biased <- renderPlot({
      x <- rnorm(n_obs(), mu(), st_dev())
      y <- c(rep(0, n_obs()-1))
      # y vai acumulando a variância à medida que o número de observações aumenta
      for (i in 2:n_obs()) {
          y[i-1] <- var(x[1:i]) * (i-1) / i
      }
      g <- ggplot(data.frame(x = 1:(n_obs()-1), y = y), aes(x = x, y = y))
      g <- g + geom_hline(yintercept = 0) + geom_line(size = 2) + geom_hline(yintercept = st_dev()^2, col="red") + geom_line(size = 2)
      g <- g + labs(x = "Número de observações", y = "Estimador tendencioso da variância acumulado")
      g
  })

  #plota o histograma da variância não tendenciosa da amostra
  output$unbiased_variance_hist <- renderPlot({
      y <- c(rep(0, n_amostras()))
      for (i in 1:n_amostras()) {
          x <- rnorm(sample_size(), mu(), st_dev())
          y[i] <- var(x)
      }
      z <- mean(y)
      h <- hist(y, xlim=c(0, max()), main = '', xlab = "variância amostral (estimador não tendencioso)", ylab = "Frequência")
      h
      i <- which.max(h$counts)
      abline(v=c(st_dev()^2, z), col=c("red", "blue"), lw = 3)
      legend(x = max()/2, y = 3*h$counts[i]/4, legend = c("Parâmetro","Média do Estimador"), fill=c("red","blue"))
  })
  
  #plota o histograma da variância tendenciosa da amostra
  output$biased_variance_hist <- renderPlot({
      y <- c(rep(0, n_amostras()))
      for (i in 1:n_amostras()) {
          x <- rnorm(sample_size(), mu(), st_dev())
          y[i] <- var(x) * (sample_size() - 1) / sample_size()
      }
      z <- mean(y)
      h <- hist(y, xlim=c(0, max()), main = '', xlab = "variância amostral (estimador tendencioso)", ylab = "Frequência")
      h
      abline(v=c(st_dev()^2, z), col=c("red", "blue"), lw = 3)
      i <- which.max(h$counts)
      legend(x = max()/2, y = 3*h$counts[i]/4, legend = c("Parâmetro","Média do Estimador"), fill=c("red","blue"))
  })
  
  }

ui <- fluidPage(
  
  titlePanel(expression("Estimadores da variância de uma distribuição")),
  
  sidebarLayout(
    sidebarPanel(
    sliderInput("n", 
                label = expression("Número de amostras"), 
                min = 10, max = 10000, step = 1,
                value = 500
    ),
    sliderInput("sample_size", 
                label = "Tamanho da amostra", 
                min = 3, max = 100, step = 1,
                value = 4
    ),
    sliderInput("mean", 
                label = expression("Média da distribuição normal"), 
                min = 0, max = 40, step = 1,
                value = 20
    ),
    sliderInput("st_dev", 
                label = expression("Desvio padrão da distribuição normal"), 
                min = 1, max = 20, step = 1,
                value = 4
    ),
    submitButton("Submeter")
    ),
    
    # Show plots
    mainPanel(
        fluidRow(
            column(4, plotOutput("cum_variance_unbiased")),
            column(4, plotOutput("cum_variance_biased"))
        ),
        fluidRow(
            column(4, plotOutput("unbiased_variance_hist")),
            column(4, plotOutput("biased_variance_hist"))
        )
    )
  )
)


shinyApp(ui = ui, server = server)