#
# applicação que traça as curvas de probabilidades pós-teste em função da prevalência (probabilidade pré-teste) 
# para valores de sensibilidade e especificidade especificados pelo usuário
# 
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# Licença: Creative Commons: CC BY-SA
#

library(shiny)

# Função que fornece o valor preditivo positivo
vpp <- function(prev, sens, spec) {
    (prev * sens) / (prev*sens + (1-spec) * (1 - prev))
}

# Função que fornece a probabilidade de ter a doença quando o teste é negativo
probD_test_negative <- function(prev, sens, spec) {
    (prev* (1 - sens) ) / (prev* (1 - sens) + spec * (1 - prev))
}

# Função que fornece a probabilidade de não ter a doença quando o teste é positivo
probNotD_test_positive <- function(prev, sens, spec) {
    (1-prev) * (1-spec) / (prev*sens + (1-spec) * (1 - prev))
}

# Função que fornece o valor preditivo negativo
vpn <- function(prev, sens, spec) {
    (1-prev) * spec  / (prev* (1 - sens) + spec * (1 - prev))
}

# Define server logic required to draw the curve
server <- function(input, output) {

#Parâmetros escolhidos pelo usuário: especificidade, sensibilidade e prevalência
  specificity <- reactive ({
    input$specificity
  })
  
  sensitivity <- reactive ({
    input$sensitivity
  })

  prevalence <- reactive ({
      input$prevalence
  })
  
 # Gráfico do valor preditivo positivo
  output$vpp_curve <- renderPlot({
    curve(vpp(x, sensitivity(), specificity()), xlim=c(0,1), main="", xlab = "", ylab = "", lwd = 2)
    par(cex=1.5)
    title(xlab = "Probabilidade Pré-teste", ylab = "Valor Preditivo Positivo")
    # determina o ponto com coordenadas (prevalencia, vpp)
    y <- vpp(prevalence(), sensitivity(), specificity())
    points(prevalence(), y, col = "blue", pch=21, cex = 2.0, bg = "blue")
    # para prevalência > 0.8, fixa a posição da abscissa do texto
    if (prevalence() > 0.8) {
        text(0.8, y -.05, paste("VPP = ", sprintf("%.2f", y)), pos = 4)
    } else {
        text(prevalence() +.01, y -.05, paste("VPP = ", sprintf("%.2f", y)), pos = 4)
    }
  })
  
  output$vpn_curve <- renderPlot({
      curve(vpn(x, sensitivity(), specificity()), xlim=c(0,1),main="", xlab = "", ylab = "", lwd = 2)
      par(cex=1.5)
      title(xlab = "Probabilidade Pré-teste", ylab = "Valor Preditivo Negativo")
      # determina o ponto com coordenadas (prevalencia, VPN)
      y <- vpn(prevalence(), sensitivity(), specificity())
      points(prevalence(), y, col = "blue", pch=21, cex = 2.0, bg = "blue")
      # para prevalência > 0.8, fixa a posição da abscissa do texto
      if (prevalence() > 0.8) {
          text(0.8, y -.1, paste("VPN = ", sprintf("%.2f", y)), pos = 4)
      } else {
          text(prevalence() +.01, y -.1, paste("VPN = ", sprintf("%.2f", y)), pos = 4)
      }
  })
  
}


# Define UI for application that draws a roc curve
ui <- fluidPage(
  
  # Application title
  titlePanel("Probabilidade Pós-Teste x Probabilidade Pré-teste"),
  
  # Sidebar with a slider input for the cutoff
  sidebarLayout(
    sidebarPanel(style = "font-size: 20px; ",
                 tags$style("[type = 'number'] {font-size:20px;height:50px;}"),
      sliderInput("sensitivity", 
                 label = "Sensibilidade", 
                 min = 0.0001, max = .9999,
                 value = 0.98
    ),
    sliderInput("specificity", 
                 label = "Especificidade", 
                 min = 0.0001, max = .9999,
                 value = 0.7
    ),
    tags$style(type = "text/css", "
              .irs-single {color:white; font-size: 20px; top: -10px}
          "),
    numericInput("prevalence", 
                label = expression("Probabilidade pré-teste"), 
                min = 0, max = 1, step = .05,
                value = 0.5)
  ),
  

  
    # Show plots
    mainPanel(
        fluidRow(
            column(8, plotOutput("vpp_curve")),
            column(8, plotOutput("vpn_curve"))
        )
    )
  )
)


shinyApp(ui = ui, server = server)